<?php
if ( ! defined( 'WPINC' ) ) {
	wp_die();
}

/**
 * Class Iconic_WAP_Pages
 */
class Iconic_WAP_Pages {
	/**
	 * @var string
	 */
	protected static $query_var = 'iconic_wap_endpoint';

	/**
	 * Run class.
	 */
	public static function initiate() {
		$subpages = Iconic_WAP_Helpers::get_account_subpages();

		if ( empty( $subpages ) ) {
			return;
		}

		add_filter( 'woocommerce_account_menu_items', array( __CLASS__, 'account_menu_items' ), 10, 1 );
		add_filter( 'woocommerce_account_menu_item_classes', array( __CLASS__, 'account_menu_item_classes' ), 10, 2 );
		add_filter( 'the_title', array( __CLASS__, 'the_title' ), 10, 2 );
		add_filter( 'query_vars', array( __CLASS__, 'register_query_var' ), 10, 1 );
		add_action( 'woocommerce_account_' . self::$query_var . '_endpoint', array( __CLASS__, 'endpoint_content' ) );
		add_action( 'template_redirect', array( __CLASS__, 'prevent_endpoint_indexing' ) );
		add_action( 'template_redirect', array( __CLASS__, 'restrict_access' ) );
		add_filter( 'woocommerce_get_breadcrumb', array( __CLASS__, 'get_breadcrumb' ), 10, 2 );
		self::add_my_account_endpoints();
	}

	/**
	 * Account menu items.
	 *
	 * @param array $items
	 *
	 * @return array
	 */
	public static function account_menu_items( $items ) {
		$subpages    = Iconic_WAP_Helpers::get_account_subpages();
		$logout_item = isset( $items['customer-logout'] ) ? $items['customer-logout'] : false;

		foreach ( $subpages as $subpage ) {
			if ( ! self::is_visible( $subpage->post_name ) ) {
				continue;
			}

			$items[ $subpage->post_name ] = $subpage->post_title;
		}

		if ( $logout_item ) {
			unset( $items['customer-logout'] );
			$items['customer-logout'] = $logout_item;
		}

		return $items;
	}

	/**
	 * Account menu item classes.
	 *
	 * @param array  $classes
	 * @param string $endpoint
	 *
	 * @return array
	 */
	public static function account_menu_item_classes( $classes, $endpoint ) {
		$endpoint_id = get_query_var( self::$query_var );

		if ( ! $endpoint_id ) {
			return $classes;
		}

		$endpoint_slug = get_post_field( 'post_name', $endpoint_id );

		if ( $endpoint_slug !== $endpoint ) {
			return $classes;
		}

		$classes[] = 'is-active';

		return $classes;
	}

	/**
	 * Add endpoint.
	 */
	protected static function add_my_account_endpoints() {
		$subpages          = Iconic_WAP_Helpers::get_account_subpages();
		$account_page_id   = get_option( 'woocommerce_myaccount_page_id' );
		$account_page_path = get_page_uri( $account_page_id );

		if ( empty( $subpages ) ) {
			return;
		}

		foreach ( $subpages as $subpage ) {
			$uri      = sprintf( '%s/%s', $account_page_path, $subpage->post_name );
			$redirect = sprintf( 'index.php?page_id=%d&%s=%d', $account_page_id, self::$query_var, $subpage->ID );

			add_rewrite_rule( $uri, $redirect, 'top' );
		}

		if ( version_compare( get_option( 'iconic_wap_rewrite_rule' ), Iconic_Woo_Account_Pages::$version, '<' ) ) {
			flush_rewrite_rules();
			update_option( 'iconic_wap_rewrite_rule', Iconic_Woo_Account_Pages::$version );
		}
	}

	/**
	 * Register query var.
	 *
	 * @param array $vars
	 *
	 * @return array
	 */
	public static function register_query_var( $vars ) {
		$vars[] = self::$query_var;

		return $vars;
	}

	/**
	 * Output endpoint content.
	 */
	public static function endpoint_content() {
		$current_endpoint_page = self::get_current_endpoint_page_object();

		if ( ! $current_endpoint_page ) {
			return;
		}

		echo apply_filters( 'the_content', $current_endpoint_page->post_content );
	}

	/**
	 * Change endpoint title.
	 *
	 * @param string $title
	 * @param int    $post_id
	 *
	 * @return string
	 */
	public static function the_title( $title, $post_id = null ) {
		if ( ! $post_id ) {
			global $post;

			$post_id = $post->ID;
		}

		$woo_account_page_id = Iconic_WAP_Helpers::get_myaccount_page_id();

		if ( is_admin() || $post_id !== $woo_account_page_id || ! in_the_loop() ) {
			return $title;
		}

		$endpoint = self::get_current_endpoint_page_object();

		if ( ! $endpoint ) {
			return $title;
		}

		return $endpoint->post_title;
	}

	/**
	 * Get current endpoint page object.
	 *
	 * @return bool|WP_Post
	 */
	protected static function get_current_endpoint_page_object() {
		$endpoint_id = self::is_endpoint();

		if ( ! $endpoint_id ) {
			return false;
		}

		$endpoint_post = get_post( $endpoint_id );

		if ( is_null( $endpoint_post ) || is_wp_error( $endpoint_post ) ) {
			return false;
		}

		return $endpoint_post;
	}

	/**
	 * Is endpoint.
	 *
	 * @return bool
	 */
	public static function is_endpoint() {
		$endpoint_id = get_query_var( self::$query_var );

		if ( ! $endpoint_id ) {
			return false;
		}

		return $endpoint_id;
	}

	/**
	 * No index endpoints.
	 *
	 * @since 1.0.2
	 */
	public static function prevent_endpoint_indexing() {
		if ( ! self::is_endpoint() ) {
			return;
		}

		@header( 'X-Robots-Tag: noindex' );
	}

	/**
	 * Restrict access.
	 *
	 * @since 1.0.2
	 */
	public static function restrict_access() {
		if ( ! self::is_endpoint() ) {
			return;
		}

		if ( is_user_logged_in() ) {
			return;
		}

		wp_safe_redirect( wc_get_page_permalink( 'myaccount' ) );
		exit;
	}

	/**
	 * Add page to breadcrumbs.
	 *
	 * @param array         $crumbs
	 * @param WC_Breadcrumb $breadcrumbs
	 *
	 * @return array
	 */
	public static function get_breadcrumb( $crumbs, $breadcrumbs ) {
		$endpoint = self::is_endpoint();

		if ( ! $endpoint ) {
			return $crumbs;
		}

		$title    = get_the_title( $endpoint );
		$crumbs[] = array( $title, '' );

		return $crumbs;
	}

	/**
	 * Is the page visible for the current user?
	 *
	 * @param $slug
	 *
	 * @return bool
	 */
	public static function is_visible( $slug ) {
		$hidden = apply_filters( 'iconic_wap_hidden_pages', array() );

		if ( empty( $hidden ) ) {
			true;
		}

		return ! in_array( $slug, $hidden );
	}
}